// -*- mode: c++; c-indent-level: 4; c++-member-init-indent: 8; comment-column: 35; -*-

// -------------------------------------vq.cc-------------------------------
// Simple unsupervised vector quantization algorithm using  GA and the
// Weight class
// vq2 is the same than vq1, sauf the fixed range chromosomes

#include "gensgav.hpp"
#include "scanfile.hpp"
#include "ezsample.hpp"
#include "gstream.hpp"


float fitness (const genSGA<float> & myGen, ezSample<float> & sample ) {
// VQ algorithm or Vector Quantization, algorithm: the distance from the
// sample to the "winning" vector must be minimized

    const unsigned vecsInGen = myGen.getLength()/2;
    // compute vectors corresponding to the genome
    Weight<float> wArray[vecsInGen]; // 1 vector = 2 comps; using default ctor

    for ( unsigned j = 0; j < vecsInGen; j++ ) {
	wArray[j] = myGen.extractVect(j*2, j*2 +1); // assignment operator
    }

    float totDist = 0.0;
    for ( unsigned i = 0; i < sample.getSize(); i ++ ) {

	float minDist = 1000;	   // much bigger that vectors in the
				   //  range [-1,1]	
	Weight<float> auxW =  sample.getVect(i) ;
	for ( j = 0; j < vecsInGen; j++ )  {
	    float thisDist = auxW.distE( wArray[j] );  
				   // implicit conversion
	    if ( thisDist < minDist ) 
		minDist = thisDist;
	}
	totDist += minDist;
				 
    }

    return 1/totDist;
}




//--------------------------------


main( int argc,  char ** argv ) {
	
    char 
      inputFile[MAXFILENAME], 
      outputFile[MAXFILENAME];	   // input and output files
    unsigned 
      popSize,		// population size
      generations,	// number of generations
      locusSize;	// size in bytes of each locus/gene
    mutRate_t mutRate;	// mutation rate; xOver is uniform with prob 0.01
    enum reprodMode thisRepMode; // reproduction and pop selection mode
    unsigned uArgRepMode;
    float fArgRepMode;		   // both arguments for reproduction mode
    mybool gnuplot;		   // plotting with gnuplot or not, dummy if !gnuplot 
    unsigned intervOutput =1;	   // generations to pass before writing anything
    mybool outBest, outFitness;	   // output fitness or best individual
    const unsigned genSize = 10;   // number of genes in the chromosome
				   // and 2 times number of vectors
    float
      rangeStart, rangeEnd;	   // ranges for each gene

    if ( argc > 1 )
      scanFile2(rangeStart, rangeEnd,  argv[1], inputFile, outputFile,
	       popSize,  mutRate,  generations, locusSize, 
	       thisRepMode, uArgRepMode, fArgRepMode, 
	       gnuplot, intervOutput, outFitness, outBest );
				   // using overload; genSize not needed
    else {
	cout << "Usage: " << argv[0] << " <inifile>.ini \n";
	exit(1);
    }


    // output file, if specified
    mybool thruStdout;
    if ( strcmp( outputFile, "STDOUT" ) == 0 )
      thruStdout = TRUE;	// output thru stdout or file;
    ofstream fout;

    if (thruStdout == FALSE) 
      fout.open( outputFile );

    mysrand();
    ezSample<float> sample( inputFile, genSize / 5);
    // population declaration
    popSGAr<float, genSGA<float> > 
      myPop(popSize, mutRate, genSize, locusSize,rangeStart,rangeEnd);

    // Only if gnuplot is installed----
    gstream gout("");
    gstream gvout("", POINTS, FALSE);
    // --------------------------------
    

    // Genetic algorithm loop----------
    cout << "Starting genetic algorithm";
    for ( unsigned g= 0; g < generations ; g++ ) {
	
	// Evaluate population fitness
	for ( unsigned p = 0; p < myPop.getPopSize(); p ++ ) 
	  myPop.setFitness( p, fitness( *(myPop[p]), sample ));

	// now really apply GA

	switch ( thisRepMode ) {
	  case ELITE:
	    myPop.newGeneration( thisRepMode, fArgRepMode ); 
	    break;
	  case TOURNAMENT:
	  case ROULWHEEL:	// argument is not used in ROULWHEEL
	    myPop.newGeneration( thisRepMode, uArgRepMode );
	    break;
	}
	
	if ( thruStdout == TRUE ) {	// equal
	    if ( outBest == TRUE ) 
	      cout << myPop.bestIndiv() << endl;
	    if (outFitness == TRUE )
	      cout << myPop.bestFitness() << endl;
	    cout.flush();
	} else {
	    if (outBest == TRUE ) 
	      fout << myPop.bestIndiv() << endl;
	    if (outFitness == TRUE )
	      fout << myPop.bestFitness() << endl;
	    fout.flush();
	}
	
	// Only if gnuplot is installed----
	if (gnuplot == TRUE ) {
	    gout << myPop.bestFitness();
	    gvout.reStart();
	    for ( unsigned i = 0; i < genSize / 2; i ++ ) {
		Weight<float> auxW;
		auxW = (myPop.bestIndiv()).extractVect(i*2, i*2 +1);
		gvout << auxW.getVec();
	    }
	}
				    
	//---------------------------------

    }
    
    // close output file, if needed 
    if (thruStdout == TRUE )
      fout.close();

    return 0;
}
