// -*- mode: c++; c-indent-level: 4; c++-member-init-indent: 8; c++-electric-colon: t; comment-column: 35 -*-

//				BITGENE.HPP
// DESCRIPTION: Inherits from rawgene, and loci are described by its 
// length in bits.
// All loci have the same length, and they are decoded to unsigned integer
//  numbers.


#ifndef	__BITGENE_HPP
#define __BITGENE_HPP


#include "rawgene.hpp"			    // base class


class bitGene: public rawGene {
					    
public:

    bitGene() {};
    virtual ~bitGene() {};
    
    // more or less random constructors
    bitGene(genLength_t _numLoci,unsigned _bitSize,
	    float dummy1=0, float dummy2=0) 
      :rawGene( ( _bitSize*_numLoci ) /BITSINBYTE + 1), 
       bitSize( _bitSize), 
       numLoci(_numLoci) {};

    bitGene(genLength_t _numLoci, unsigned _bitSize,
	    void* dummy1, void* dummy2)
      :rawGene( ( _bitSize*_numLoci ) /BITSINBYTE + 1), 
       bitSize( _bitSize), 
       numLoci(_numLoci) {};               // similar to genSGA[v], dummy ctor
    
	
    // "binary" constructors, from two loving parents
    bitGene(const bitGene &bitGene1, 
	    const bitGene &bitGene2, 
	    genLength_t _startPoint, 
	    genLength_t _endPoint, 
	    mutRate_t _mutationRate)
      : rawGene( bitGene1, bitGene2, _startPoint, _endPoint,  _mutationRate), 
        bitSize( bitGene1.bitSize ), 
        numLoci( bitGene1.numLoci ) {};	// 2-point crossover

     bitGene(const bitGene &_bitGene1, 
	     const bitGene &_bitGene2, 
	     xOverRate_t _xOverRate, 
	     mutRate_t _mutationRate = 0)
      : rawGene( _bitGene1, _bitGene2, _xOverRate, _mutationRate), 
        bitSize( _bitGene1.bitSize ), 
        numLoci( _bitGene1.numLoci ) {};	// 2-point crossover
    
    // "unary" constructors, from batchelor parent or from an amoeba
    // including transposition
    bitGene(const bitGene& bitGene1, 
	    mutRate_t _mutationRate, 
	    genLength_t _genTr1= 0, 
	    genLength_t _genTr2 = 0); 
				   // Transposition not done by default

    bitGene(const bitGene &_bitGene, 
	   genOpMode _modeOp, 
	   genLength_t _geneIdx = 0, 
	   mutRate_t _mutationRate = 0,
	   genLength_t _geneIdx2 = 0);	
				   // creates a new genome using unary
				   // genetic operators; using _geneIdx
				   // if needed and _mutationRate; KILL,
				   // RANDINC and DUP have no effect

    // copy constructor
    bitGene(const bitGene& newBitGene )	
      : rawGene( newBitGene ),	   // Calls rawGene copy-ctor
        bitSize( newBitGene.bitSize ),
        numLoci( newBitGene.numLoci ) {};
	
    // info functions
    virtual unsigned getLength() const { return numLoci; };
				   // "Length" is the number of possible
				   // genome indices

    unsigned bit (unsigned _idx ) const; 
				   // _idx is the bit index, returns 0/1
    unsigned operator [] ( unsigned _idx ) const { return getValue( _idx );};
				   // returns gene value

    // standard conversion + ctor
    operator unsigned* ( void ) const;
    operator char*     ( void ) const;
    

    // "friendly" functions
    friend ostream& operator << ( ostream& s, const bitGene & _inGen );
	
	
private:
    unsigned bitSize;		   // number of bits per locus, 
    genLength_t numLoci;	   // number of loci

    unsigned getValue( unsigned _idx ) const;
    
};

#endif				   // __BITGENE_HPP
