// -*- mode: c++; c-indent-level: 4; c++-member-init-indent: 8; comment-column: 35 -*-


//			GENSGAV.HPP
// DESCRIPTION: same as gensga.hpp, but with variable range for each locus
//		in the cromosome.


#ifndef __GENSGAV_HPP
#define __GENSGAV_HPP

#ifndef __BCPLUSPLUS__
#ident "@(#) gensgav.hpp -- by J. J. Merelo"
#endif

#include "genvar.hpp"
#include "weight.hpp"


template <class returnType>
class genSGAv: public genVar {


public:


    // constructors and destructors.
    virtual ~genSGAv () {
	delete[] rangeA;
	delete[] rangeStartA;
    }
    
    genSGAv();
    genSGAv(genLength_t _numAlleles, 
	    unsigned _sizeAllele, 
	    returnType *_rgStart=0, 
	    returnType *_rgEnd=0 );	     // 0 is for dummy purposes
 
    genSGAv(genLength_t _numAlleles, 
	    unsigned _sizeAllele, 
	    returnType _rgStart=0, 
	    returnType _rgEnd=0 );	     // ctor similar to genSGA
	    
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
//               Binary Ctors
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   
    genSGAv(const genSGAv &_genSGAv1, 
	    const genSGAv &_genSGAv2, 
	    genLength_t _startPoint, genLength_t _endPoint, 
	    mutRate_t _mutationRate)
      :genVar( _genSGAv1, _genSGAv2, _startPoint, 
	      _endPoint, _mutationRate ) { 
	  allocRanges2( _genSGAv1.rangeStartA, _genSGAv1.rangeA );	
      };			   // 2-pt xOver + mutation


    genSGAv(const genSGAv &_genSGAv1, 
	    const genSGAv &_genSGAv2, 
	    xOverRate_t _xOverRate, 
	    mutRate_t _mutationRate = 0) 
      :genVar(_genSGAv1, _genSGAv2, 
	      _xOverRate, _mutationRate ) { 
	  allocRanges2( _genSGAv1.rangeStartA, _genSGAv1.rangeA );	
      };			   // Uniform xOver + opt. mutation


//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
//               Unary constructors, from only 1 parent
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

    genSGAv(const genSGAv &_genSGAv, // creates a new genome using one
	    genOpMode _modeOp,	     // of 3 modes; using _geneIdx if
	    genLength_t _geneIdx = 0, // needed and _mutationRate
	    mutRate_t _mutationRate = 0,
	   genLength_t _geneIdx2 = 0 )
      :genVar( _genSGAv, _modeOp, _geneIdx, _mutationRate, _geneIdx2 ){ 
	  allocRanges2( _genSGAv.rangeStartA, _genSGAv.rangeA );
      };	
    
    genSGAv( const genSGAv &_genSGAv, // straightforward clone
	   mutRate_t _mutationRate )
      :genVar( _genSGAv, _mutationRate ) 
	{
	    allocRanges2( _genSGAv.rangeStartA, _genSGAv.rangeA );
	    
	}
        
    genSGAv(const genSGAv& _inGen );	     // copy constructor
  

// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- 
//   Common functions to all the "gene" hierarchy
// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

    // "info" functions, and overloaded operators
    returnType	operator [] (genLength_t _idx) const;

    // getLength is directly inherited from genVar

    // conversion operators
    operator returnType * ( void ) const;
    operator Weight<returnType> (void) const;
    Weight<returnType> extractVect(unsigned _idx1, unsigned _idx2) const;

    // Particular info function
    // now functions for fancy Gray decoding
    returnType	deGray	    ( genLength_t _idx ) const;
   
    // Friend declaration
friend ostream& operator << ( ostream& s, const genSGAv<returnType> & _inGen );
    

private:

    returnType *rangeStartA, *rangeA; // beginning of range and range
	
	
    void allocRanges(returnType *_rgStart, 
		     returnType *_rgEnd );
	    
    void allocRanges2(returnType *_rgStart, 
		      returnType *_rg );
				   // same as before, but using real ranges
						
	    
};

#include "gensgav.icc"		   // templates and inlines, hidden
				   // from prying eyes


#endif
