// -*- mode: c++; c-indent-level: 4; c++-member-init-indent: 8; c++-electric-colon 1; comment-column: 35  -*-

// GENVAR.HPP : Variable length genotype base class
//              But lately descends from rawgene.hpp

#ifndef __GENVAR_HPP
#define __GENVAR_HPP

#ifndef __BCPLUSPLUS__
#ident "@(#) genvar.hpp -- By JJ Merelo "
#endif

#include "general.hpp"
#include "rawgene.hpp"

// useful but not class functions
unsigned char deGray( unsigned char _deG ); // converts from Gray to binary


// ---------------------------------------------------------------------------
// -------------------------- Class definition -------------------------------

class genVar: public rawGene {


public:
    // several constructors, and one destructor

    genVar( void ) {}; 		// default constructor


    // more or less random constructors
    genVar(genLength_t _numAlleles, 
	   unsigned _sizeAllele );

    genVar(genLength_t _numAlleles,
	   const char *_str, 
	   unsigned _sizeAllele )
      :rawGene( _numAlleles*_sizeAllele, _str ) {
	  initSize( _sizeAllele );
      };
	
    genVar(const char *_str, 
	   genLength_t _length, 
	   mutRate_t _mutationRate )
      :rawGene( _str, _length, _mutationRate ) {
	initSize( 1 );
      };
	
	
    // "binary" constructors, from two loving parents
    genVar(const genVar &_genVar1, 
	   const genVar &_genVar2,
	   xOverRate_t _xOverRate, 
	   mutRate_t _mutationRate = 0)
      :rawGene( _genVar1, _genVar2, _xOverRate, _mutationRate ) { 
	initSize( _genVar1.getSize() ); 
      };			// normal reproduction, with uniform xOver
					

    genVar(const genVar &genVar1, 
	   const genVar &genVar2,
	   xOverRate_t xOverRate, mutRate_t MutationRate, 
	   killRate_t killRate, incRate_t dupRate );


    genVar(const genVar & _genVar1, 
	   const genVar & _genVar2, 
	   genLength_t _startPoint, 
	   genLength_t _endPoint, 
	   mutRate_t _mutationRate)
      :rawGene( _genVar1,  _genVar2, _startPoint, _endPoint,_mutationRate ) { 
	initSize( _genVar1.getSize() ); 
      };				// 2-point crossover
	
	
    // "unary" constructors, from batchelor parent or from an amoeba
    genVar(const genVar &genVar1, // Cloning with mutation ctor.
	   mutRate_t MutationRate, 
	   killRate_t killRate = 0 , incRate_t dupRate = 0);
	
    genVar(const genVar &genVar1, 
	   genOpMode _modeOp, 
	   genLength_t _geneIdx = 0, 
	   mutRate_t _mutationRate = 0,
	   genLength_t _geneIdx2 = 0);	
				// creates a new genome using unary
				// genetic operators; using _geneIdx
				// if needed and _mutationRate
    // Copy constructor
    genVar( const genVar &_inGen )
      :rawGene( _inGen ) {
	initSize( _inGen.getSize() );
      }

    // destructor
    ~genVar() {};
	
    // common interface
    virtual unsigned getLength() const { return getNumAlleles();}

    // Friend declaration
    friend ostream& operator << ( ostream& s, const genVar & _inGen );


// -------------------------------------------------------------------

protected:


    // protected methods
    void initSize( unsigned _sizeAllele);
    // underground constructor
					
	
    // protected info functions
    unsigned long getMax4Size( void ) const { return maxForSize;};
    unsigned	  getSize() const {return size;} ;
    unsigned 	  getNumAlleles( void ) const { 
      return getNumBytes()/size;
    } ;

    // other functions
	
    void 	  getAllele( genLength_t idx, char* _inStr ) const;
    const char*	  getAllele( genLength_t idx) const;

    // variable length functions
    void 	  addAllele (void );
    void 	  killAllele(genLength_t _pos );
    void 	  dupAllele (genLength_t _pos, 
			     mutRate_t MutationRate );

// --------------------------------------------------------------------

private:
    
    unsigned size;		// size of each allele, in bytes
    unsigned long maxForSize;   // max number code-able with this locus size
				// *WARNING* Number of bytes for this depends
				// on the OS and compiler
	
};

typedef genVar* pGenVar;


#endif
