// -*- mode: c++; c-indent-level: 4; c++-member-init-indent: 8; c++-electric-colon: t; comment-column: 35 -*-

#include "scanfile.hpp"

// Non- class functions, non-template

//--------------------------------------------------------------------------
void scanBest( const char* _inStr, mybool & _fit, mybool & _best ) {
//--------------------------------------------------------------------------

    _fit = _best = FALSE;
    // Check for F, V and B
    if ( (strchr( _inStr , 'F') != 0)||( strchr( _inStr , 'f' ) != 0 ) )
      _fit = TRUE;
    if ( (strchr( _inStr , 'V') != 0)||( strchr( _inStr , 'v' ) != 0 ) )
      _best = TRUE;
    if ( (strchr( _inStr , 'B') != 0)||( strchr( _inStr , 'B' ) != 0 )){
	_best = TRUE;
	_fit = TRUE;
    }
}

//______________________________________________________
void scanRep( const char* _inStr, 
	     enum reprodMode& _rep,
	     float& _fArg,
	     unsigned& _uArg ) {
//______________________________________________________

    istrstream istr( _inStr, MAXPARAMSIZE );
    char tmpRepStr[MAXPARAMSIZE];
    istr >> tmpRepStr;

    if ( strcmp( tmpRepStr, "ELITE" ) == 0 ) {
	_rep = ELITE;
	istr >> _fArg;
	if (_fArg > 1) { // It's percentage over 1
	    cerr << "ELITE reproduction argument must be 0 < arg < 1\n";
	    exit(1);
	}
    }
    else if  ( strcmp( tmpRepStr, "TOURNAMENT" ) == 0 ) {
	_rep = TOURNAMENT;
	istr >> _uArg;
    }
    else if  ( strcmp( tmpRepStr, "ROULWHEEL" ) == 0 )
      _rep = ROULWHEEL;
    
}


//--------------------------------------------------------------------------
void scanFile3( char* _fileN,		
	       char* _inputFile, 
	       char* _outputFile,
	       unsigned& _popSize,
	       mutRate_t& _mutRate,
	       unsigned& _gens,
	       enum reprodMode & _repType,  
	       unsigned& _uRepArg,
	       float&    _fRepArg,
	       mybool& _gnuplot,
	       unsigned& _outputInterv,
	       mybool& _outFitness,
	       mybool& _outBest ) { 

// -- scan files for commands of the following form
//    description parameter
//    description is a symbolic name; while parameter is a string or numeric
//    value. All values are returned in vars passed by reference
//    It's quite ad hoc, but it works.
//    Actually, it looks only for the first letter. So, no more than 24
//    parameters.
// -------------------------------------------------------------------------

    const unsigned MAXLINESP = 100;
    char tmpStr[MAXLINESP];
    paramArr params( _fileN ); // mutRate is float

    // output options
    char auxfv[MAXLINESP];
    params.getStr( 'B', auxfv, MAXLINESP );
    scanBest( auxfv, _outFitness, _outBest );

    params.getStr( 'F', _outputFile, MAXFILENAME );
    _gens = params.getInt( 'G' );
    params.getStr('I', _inputFile, MAXFILENAME );
    _mutRate = params.getFloat( 'M' );
    _outputInterv = params.getInt( 'O' );
    _popSize    = params.getInt( 'P' );
    
    // reproduction type
    params.getStr( 'R', tmpStr, MAXLINESP  );
    scanRep( tmpStr, _repType, _fRepArg, _uRepArg );

    // use gnuplot or not
    params.getStr( 'W', tmpStr, MAXLINESP  );
    _gnuplot = (toupper(tmpStr[0]) == 'Y')?TRUE:FALSE;

}

