// -*- mode: c++; c-indent-level: 4; c++-member-init-indent: 8; comment-column: 35; -*-

#ifndef __WEIGHT_HPP
#define __WEIGHT_HPP

#ifndef __BCPLUSPLUS__
#ident "@(#) weight.hpp -- by J. J. Merelo"
#endif

#include <stdio.h>		   // for exit

#include <fstream.h>		   // for ofstream

#include "general.hpp"		   // for mybool

//____________________ Vector class ___________________________________
// Class Vector: this is a simple implementation of a "Vector" template

template <class vecType>
class Weight{

  public:
    
    // Constructors from already defined vectors, and without them
    Weight	   	( unsigned _dim, const vecType *_inVec = NULL );

    // default constructor
    Weight		( void ):Vec(0),Dim(0) {};
    
    // Copy constructor
    Weight		( const Weight& _weight );

    // overloading of assignment operator
    const Weight & operator = ( const Weight& _weight );
    const Weight & operator = ( const vecType * _inVec );

    // another assignment/conversion/construction operator
    operator vecType* ( void ) const;

    // Vector distances
    inline vecType distE   ( const Weight& _weight ) const;// euclidea
    vecType 	   distM   ( const Weight& _weight ) const;// del taxista, o Manhattan

    // Operadores sobrecargados; referencia: Stroustrup, p 301. Dice que es
    // as, y yast, coo. -- More overloaded operators; ref
    // Stroustrup, p 301. Because he says so, and 'nuff said.

    Weight& operator += ( const Weight& _weight );
    Weight& operator -= ( const Weight& _weight );

    // operadores sobrecargados. Seguiremos, como el siguiente, el Stroustrup
    Weight  operator + 	( const Weight& _weight );
    Weight  operator - 	( const Weight& _weight );

    Weight  operator * ( float _alfa ); // scalar product
    Weight  operator / 	( long _alfa );	// integer division

    // operadores para acceder al interior. Sirve tambien como l-value
    vecType& operator[] 	( unsigned _elem )  const {
	return (_elem<Dim)?Vec[_elem]:Vec[Dim];	// Checks limits
    }

    // dimension operators
    unsigned getDim	(void) const { return Dim; };

    // info operator
    const vecType *getVec(void) const { return Vec; };

    //	class destructor
    ~Weight(void) { delete []Vec;} ;

// friend declarations, mainly for output
friend ostream& operator << ( ostream& s, const Weight<vecType> & _inWgt );
friend ofstream& operator << ( ofstream& s, const Weight<vecType> & _inWgt );

    // seudo constructor, in case void constructor has been called
    unsigned setDim	(unsigned _dim)  {
	if (!(Vec = new vecType[Dim=_dim]))
	  return 0;
	else
	  return 1;
    };

  private:
    vecType *Vec;		   // Vector elements
    unsigned Dim;		   // Vector dimension; number of elements

    mybool dimensionCheck ( const Weight& _weight ) const; 
				   // Checks that both dimensions are equal
    
};

#include "weight.icc"		   // Template definition

#endif				   // __WEIGHT_HPP
